using System;
using System.ServiceModel;
using System.ServiceModel.Channels;
using Autoscribe.Lyons.GeneralService;
using Autoscribe.Lyons.RTNService;


namespace RtnDemo.Wrapper
{
    public class RTNServiceWrapper : IRTNServiceWrapper
    {
        private const string CfgRtnSvcEndPointConfigurationName = "RTNServiceRest";
        public LogonResult Logon(int companyId, string username, string password)
        {
            LogonResult result = null;
            try
            {
                using (ChannelFactory<IRTNService> fact = new ChannelFactory<IRTNService>(CfgRtnSvcEndPointConfigurationName))
                {
                    IRTNService svc = fact.CreateChannel();
                    result = svc.Logon(companyId, username, password);
                }
            }

            catch (FaultException fe)
            {
                result = ParseErrorResult<LogonResult>(fe);
                Console.WriteLine(" -->Error: {0}", result.ErrorMesssage);
            }
            catch (Exception ex)
            {
                result = new LogonResult(ex.Message);
                Console.WriteLine(" --> Internal Error: {0} and logon failed for company id {1} and user name {2}", ex.Message, companyId, username);
            }
            return result;
        }
        public void Logoff(string token)
        {
            try
            {
                using (ChannelFactory<IRTNService> fact = new ChannelFactory<IRTNService>(CfgRtnSvcEndPointConfigurationName))
                {
                    IRTNService svc = fact.CreateChannel();
                    svc.Logoff(token);
                }

            }
            catch (FaultException fe)
            {

                ErrorResult errorData = ParseErrorResult<ErrorResult>(fe);
                Console.WriteLine(" -->Error: {0}", errorData.ErrorMesssage);
            }
            catch (Exception ex)
            {
                Console.WriteLine(" --> Internal Error: {0}", ex.Message);
            }
        }
        public BooleanResult VerifyRTN(string token, string rtn, string country, bool includeInactive, bool includeHistory)
        {
            BooleanResult result = null;
            try
            {
                using (ChannelFactory<IRTNService> fact = new ChannelFactory<IRTNService>(CfgRtnSvcEndPointConfigurationName))
                {
                    IRTNService svc = fact.CreateChannel();
                    result = svc.VerifyRTN(token, rtn, country, includeInactive, includeHistory);
                }
            }
            catch (FaultException fe)
            {
                result = ParseErrorResult<BooleanResult>(fe);
                Console.WriteLine(" -->Error: {0}", result.ErrorMesssage);
            }
            catch (Exception ex)
            {
                result = new BooleanResult(ex.Message);
                Console.WriteLine($" --> Internal Error: {ex.Message} for parameters {rtn}, {country}, {includeInactive},{includeHistory}");
            }
            return result;
        }
        public BooleanResult VerifyWire(string token, string rtn, string country, bool includeInactive, bool includeHistory)
        {
            BooleanResult result = null;
            try
            {
                using (ChannelFactory<IRTNService> fact = new ChannelFactory<IRTNService>(CfgRtnSvcEndPointConfigurationName))
                {
                    IRTNService svc = fact.CreateChannel();
                    result = svc.VerifyWire(token, rtn, country, includeInactive, includeHistory);
                }
            }
            catch (FaultException fe)
            {
                result = ParseErrorResult<BooleanResult>(fe);
                Console.WriteLine(" -->Error: {0}", result.ErrorMesssage);
            }
            catch (Exception ex)
            {
                result = new BooleanResult(ex.Message);
                Console.WriteLine($" --> Internal Error: {ex.Message} for parameters {rtn}, {country}, {includeInactive},{includeHistory}");
            }
            return result;
        }
        public InstitutionListResult GetInstitutionsDetailsPrimaryFirst(string token, string rtn, string country, bool includeInactive, bool includeWireSearch, bool includeHistory)
        {
            InstitutionListResult result = null;
            try
            {
                using (ChannelFactory<IRTNService> fact = new ChannelFactory<IRTNService>(CfgRtnSvcEndPointConfigurationName))
                {
                    IRTNService svc = fact.CreateChannel();
                    result = svc.GetInstitutionsDetailsPrimaryFirst(token, rtn, country, includeInactive, includeWireSearch, includeHistory);
                }
            }
            catch (FaultException fe)
            {
                result = ParseErrorResult<InstitutionListResult>(fe);
                Console.WriteLine(" -->Error: {0}", result.ErrorMesssage);

            }
            catch (Exception ex)
            {
                result = new InstitutionListResult(ex.Message);
                Console.WriteLine($" --> Internal Error: {ex.Message} for parameters {rtn}, {country}, {includeInactive}, {includeWireSearch}{includeHistory}");
            }
            return result;
        }

        public InstitutionListResult GetInstitutionsDetailsBySwift(string token, string swift, string country, bool includeInactive, bool includeHistory)
        {
            InstitutionListResult result = null;
            try
            {
                using (ChannelFactory<IRTNService> fact = new ChannelFactory<IRTNService>(CfgRtnSvcEndPointConfigurationName))
                {
                    IRTNService svc = fact.CreateChannel();
                    result = svc.GetInstitutionsDetailsBySwift(token, swift, country, includeInactive, includeHistory);
                }
            }
            catch (FaultException fe)
            {
                result = ParseErrorResult<InstitutionListResult>(fe);
                Console.WriteLine(" -->Error: {0}", result.ErrorMesssage);
            }
            catch (Exception ex)
            {
                result = new InstitutionListResult(ex.Message);
                Console.WriteLine($" --> Internal Error: {ex.Message} for parameters {swift}, {country}, {includeInactive},{includeHistory}");
            }
            return result;
        }

        public InstitutionResult GetPrimaryInstitutionDetails(string token, string rtn, string country, bool includeInactive, bool includeWireSearch,
            bool includeHistory)
        {
            InstitutionResult result = null;
            try
            {
                using (ChannelFactory<IRTNService> fact = new ChannelFactory<IRTNService>(CfgRtnSvcEndPointConfigurationName))
                {
                    IRTNService svc = fact.CreateChannel();
                    result = svc.GetPrimaryInstitutionDetails(token, rtn, country, false, false, includeHistory);
                }
            }
            catch (FaultException fe)
            {
                result = ParseErrorResult<InstitutionResult>(fe);
                Console.WriteLine(" -->Error: {0}", result.ErrorMesssage);
            }
            catch (Exception ex)
            {
                result = new InstitutionResult(ex.Message);
                Console.WriteLine($" --> Internal Error: {ex.Message} for parameters {rtn}, {country}, {includeInactive}, {includeWireSearch} {includeHistory}");
            }
            return result;
        }

        public InstitutionResult GetInstitutionDetailsById(string token, int institutionId, bool includeHistory)
        {
            InstitutionResult result = null;
            try
            {
                using (ChannelFactory<IRTNService> fact = new ChannelFactory<IRTNService>(CfgRtnSvcEndPointConfigurationName))
                {
                    IRTNService svc = fact.CreateChannel();
                    result = svc.GetInstitutionDetailsById(token, institutionId, includeHistory);
                }
            }
            catch (FaultException fe)
            {
                result = ParseErrorResult<InstitutionResult>(fe);
                Console.WriteLine(" -->Error: {0}", result.ErrorMesssage);
            }

            catch (Exception ex)
            {
                result = new InstitutionResult(ex.Message);
                Console.WriteLine($" --> Internal Error: {ex.Message} for parameters {institutionId} {includeHistory}");
            }
            return result;
        }

        private static T ParseErrorResult<T>(FaultException fe) where T : ErrorResult
        {
            T res = (T)typeof(T).GetConstructor(Type.EmptyTypes)?.Invoke(null);
            MessageFault mf = fe.CreateMessageFault();
            ErrorResult errorData;
            if (mf.HasDetail)
            {
                errorData = mf.GetDetail<ErrorResult>();
            }
            else
            {
                errorData = new ErrorResult($"Unknown error: {fe.Code.Name} - {fe.Message}");
            }

            if (res != null)
            {
                res.ErrorMesssage = errorData.ErrorMesssage;
            }
            return res;
        }

    }
}
