using System;
using System.Configuration;
using Autoscribe.Lyons.GeneralService;
using Autoscribe.Lyons.RTNService;
using RtnDemo.Wrapper;

namespace RtnDemo
{
    public class DemoService
    {
        private const string CfgLogonUsername = "userName";
        private const string CfgLogonCompanyId = "companyID";
        private const string CfgLogonPassword = "userPassword";
        private const string CfgTestRtn = "demoRTN";
        private const string CfgTestCountry = "demoCountry";
        private const string CfgTestInstitutionId = "demoInstitutioId";
        private const string CfgTestSwift = "demoSwift";

        private readonly IRTNServiceWrapper rtnService = new RTNServiceWrapper();

        private string GetAppSetting(string key)
        {
            string res = null;
            try
            {
                res = ConfigurationManager.AppSettings[key];
            }
            catch (Exception ex)
            {
                Console.WriteLine($"---> ERROR at GetAppSetting(string key) with {key}: [{ex.Message}]");
            }
            return res;
        }

        public bool VerifyAll()
        {
            bool status = false;
            string rtn = GetAppSetting(CfgTestRtn);
            string country = GetAppSetting(CfgTestCountry);
            string token = Logon();
            if (token == null)
            {
                return status;
            }

            // Verify RTN end point
            BooleanResult result = rtnService.VerifyRTN(token, rtn, country, false, true);
            if (result == null)
            {
                //VerifyRTN Service call has already printed the error message 
                return status;
            }

            Console.WriteLine(result.ErrorMesssage == null
                ? $"VerifyRTN: {rtn} rtn is {result.Value}"
                : $"VerifyRTN: {rtn} rtn has error {result.ErrorMesssage}");

            // VerifyWire end point
            BooleanResult verifyWire = rtnService.VerifyWire(token, rtn, country, false, true);
            if (verifyWire == null)
            {
                // Service call has already printed the error message 
                return status;
            }
            if (verifyWire.ErrorMesssage == null)
            {
                Console.WriteLine($"VerifyWire: {rtn} rtn is {verifyWire.Value}");
                status = verifyWire.Value;
            }

            else
            {
                Console.WriteLine($"VerifyWire: {rtn} rtn has error {verifyWire.ErrorMesssage}");
                status = verifyWire.Value;
            }

            Logoff(token);
            return status;
        }

        public bool GetInstitutionsLists()
        {
            bool status = false;
            string rtn = GetAppSetting(CfgTestRtn);
            string country = GetAppSetting(CfgTestCountry);
            string token = Logon();
            string swift = GetAppSetting(CfgTestSwift);
            if (token == null)
            {
                return status;
            }
            // Test For all the rest service end point

            // GetInstitutionsPrimaryFirstDetails end point
            var institutionListResult = rtnService.GetInstitutionsDetailsPrimaryFirst(token, rtn, country, false, false, false);
            if (institutionListResult == null)
            {
                // Error is already printed inside service call for GetInstitutionsDetailsPrimaryFirst.
                return status;
            }

            if (institutionListResult.ErrorMesssage == null)
            {
                // limiting the result validation to 10 institutions
                var listCount = institutionListResult?.InstitutionList?.Count > 10 ? 10 : institutionListResult?.InstitutionList?.Count;
                for (int i = 0; i < listCount; i++)
                {
                    Institution item = institutionListResult.InstitutionList[i];
                    Console.WriteLine(
                        $"  GetInstitutionsPrimaryFirstDetails({i}): [{item.InstitutionId}] {item.Name} | {item.City} | {item.State}");
                }
            }
            else
            {
                Console.WriteLine(
                    $" GetInstitutionsPrimaryFirstDetails({rtn}): has error {institutionListResult.ErrorMesssage}");
            }

            // GetInstitutionsDetailsBySwift end point
            var institutionListResultBySwift = rtnService.GetInstitutionsDetailsBySwift(token, swift, country, false, false);
            if (institutionListResultBySwift == null)
            {
                // Error is already printed inside service call for GetInstitutionsDetailsBySwift.
                return status;
            }

            if (institutionListResultBySwift.ErrorMesssage == null)
            {
                // limiting the result validation to 10 institutions
                var listCount = institutionListResultBySwift?.InstitutionList?.Count > 10 ? 10 : institutionListResult?.InstitutionList?.Count;
                for (int i = 0; i < listCount; i++)
                {
                    if (institutionListResultBySwift.InstitutionList != null)
                    {
                        Institution item = institutionListResultBySwift.InstitutionList[i];
                        Console.WriteLine(
                            $"  GetInstitutionsDetailsBySwift({i}): [{item.InstitutionId}] {item.Name} | {item.City} | {item.State}");
                    }
                }
            }
            else
            {
                Console.WriteLine(
                    $" GetInstitutionsDetailsBySwift({rtn}): has error {institutionListResultBySwift.ErrorMesssage}");
            }

            // GetInstitutionsDetails end point
            var institutionDetails = rtnService.GetPrimaryInstitutionDetails(token, rtn, country, false, false, false);
            if (institutionDetails == null)
            {
                // Error is already printed inside service call for GetInstitutionsDetails.
                return status;
            }

            if (institutionDetails.ErrorMesssage == null)
            {
                Institution item = institutionDetails.Institution;
                Console.WriteLine(
                    $"  GetInstitutionsDetails(): [{item.InstitutionId}] {item.Name} | {item.City} | {item.State}");
                foreach (var routingNumber in item.RoutingNumbers)
                {
                    Console.WriteLine(
                        $"  >{routingNumber.RoutingNumber} | {routingNumber.Purpose}");
                    if (routingNumber.History != null)
                    {
                        foreach (var routHist in routingNumber.History)
                        {
                            Console.WriteLine(" -{0} | {1}", routHist.RoutingNumber, routHist.Purpose);
                        }
                    }
                }
            }
            else
            {
                Console.WriteLine(
                    $" GetInstitutionsDetails({rtn}): has error {institutionDetails?.ErrorMesssage}");
            }



            // GetPrimaryInstitutionWithWireDetails rest end point
            var institutionWithWireDetails = rtnService.GetPrimaryInstitutionDetails(token, rtn, "US", false, true, includeHistory: false);
            if (institutionWithWireDetails == null)
            {
                // Error is already printed inside service call for GetPrimaryInstitutionDetails.
                return status;
            }
            if (institutionWithWireDetails.ErrorMesssage == null)
            {
                Institution item = institutionWithWireDetails.Institution ?? new Institution();
                Console.WriteLine(
                    $"  GetPrimaryInstitutionWithWireDetails(): [{item.InstitutionId}] {item.Name} | {item.City} | {item.State}");
            }
            else
                Console.WriteLine(
                    $" GetPrimaryInstitutionWithWireDetails({rtn}): has error {institutionWithWireDetails.ErrorMesssage}");


            // GetPrimaryInstitutionDetails rest end point
            var primaryInstitutionDetails = rtnService.GetPrimaryInstitutionDetails(token, rtn, country, false, false, includeHistory: false);
            if (primaryInstitutionDetails == null)
            {
                // Error is already printed inside service call for GetPrimaryInstitutionDetails.
                return status;
            }
            if (primaryInstitutionDetails.ErrorMesssage == null)
            {
                Institution item = primaryInstitutionDetails.Institution ?? new Institution();
                Console.WriteLine(
                    $"  GetInstitutionsWithWireDetails({item.InstitutionId}): [{item.Name}] {item.City} | {item.State} ");
            }
            else
                Console.WriteLine(
                    $" GetInstitutionsWithWireDetails({rtn}): has error {primaryInstitutionDetails.ErrorMesssage}");
            Logoff(token);
            return status;
        }

        public bool GetInstitutionById()
        {
            bool status = false;
            string token = Logon();
            if (token == null)
            {
                return status;
            }

            int institutionId = 0;
            if (!int.TryParse(GetAppSetting(CfgTestInstitutionId), out institutionId))
            {
                Console.WriteLine("Error parsing demoInstitutioId");
                return status;
            }

            var institution = rtnService.GetInstitutionDetailsById(token, institutionId, true);
            if (institution == null)
            {
                // Error is already printed inside service call for GetInstitutionDetailsById.
                return status;
            }
            if (institution.ErrorMesssage != null)
            {
                Console.WriteLine($"Error Getting Institution details with message : {institution.ErrorMesssage}");
            }
            else
            {
                Institution item = institution.Institution;
                if (item != null)
                {
                    Console.WriteLine(
                        $"  GetInstitutionsWithWireDetails({1}): [{item.InstitutionId}] {item.Name} | {item.City} | {item.State}");
                    status = true;
                }
            }
            Logoff(token);
            return status;
        }

        private string Logon()
        {
            string userName = GetAppSetting(CfgLogonUsername);
            string password = GetAppSetting(CfgLogonPassword);
            int companyId = 0;
            if (!int.TryParse(GetAppSetting(CfgLogonCompanyId), out companyId))
            {
                Console.WriteLine("Error parsing companyID");
                return null;
            }
            string token = null;
            var res = rtnService.Logon(companyId, userName, password);
            if (res == null)
            {
                //Service printed the error.
                return null;
            }
            if (res.ErrorMesssage != null)
            {
                Console.WriteLine($"Error logging in: {res.ErrorMesssage}");
            }
            else
            {
                token = res.Token;
            }
            return token;
        }

        private void Logoff(string token)
        {
            rtnService.Logoff(token);
        }
    }
}
